/* Example: solving f(x) = 0 by the bisection method */

/* Given a nonlinear function f(x), a value of x is sought such that 
f(x) = 0. At each iteration the interval containing the root is 
halved. A root in the interval is indicated by a change in the sign 
of f(x). */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int sign(double z);  /* find sign of z */
double f(double x);  /* the function f(x) */
double bisect(double, double, double, int); /* see definition */


int main(void)
{
  double x;

  /* Find root of f(x) = 0, 0 < x < 3, to tolerance of +/- 0.000001, 
  reporting progress: */
  
	x = bisect(0.0, 3.0, 1e-6, 1);
	printf("\nRoot found: x = %9.7lf\n", x);
	
	return EXIT_SUCCESS;
}


double f(double x)  /* put your own function here */
{
	return x*x*x + 2*x*x + x - 4;  /* x == 1 is a root */
}


double bisect(              /* returns the root x found by bisection */
              double xmin,  /* x lies between xmin... */
              double xmax,  /* and xmax */
              double xtol,  /* tolerance on x */
              int verbose)  /* if true, reports progress */
{
  int it = 0;
	double xmid, y1, y2, y3;

	/* Check for a valid interval: */
	
	if (xmin >= xmax)
	{
	  puts("Error: interval incorrectly specified.");
		exit(EXIT_FAILURE);
	}

  /* Evaluate function at endpoints and check for sign change: */

  y1 = f(xmin); y2 = f(xmax);
  
	if (sign(y1) == sign(y2))
	{
	  puts("Error: single zero not located in the interval specified.");
		exit(EXIT_FAILURE);
	}

	/* Main loop: */
	
	while (1)
	{
	    if (verbose)
      printf("Iteration %i: root lies between %9.7lf and %9.7lf\n",
             it++, xmin, xmax);
      
	  /* Bisect interval, evaluate function at midpoint: */

	  xmid = (xmax + xmin)/2.0; y3 = f(xmid);

	  /* If converged, return solution: */

		if (fabs(xmax - xmin) <= xtol)
		  return xmid;
		
		/* Select the half-interval containing the zero: */

	  else if (sign(y3) == sign(y2))
		{
		  xmax = xmid; y2 = y3;
		}
	  else
		{
		  xmin = xmid; y1 = y3;
		}
  }
}


int sign(double z)  /* find sign of z */
{
  return (z >= 0) ? 1: -1;
}
